@@ -1,15 +1,31 @@
-# *-* coding: utf-8 *-*
+# author   : Johann-Mattis List
+# email    : mattis.list@gmail.com
+# created  : 2013-03-08 17:30
+# modified : 2013-03-08 17:30
 """
+Module for the derivation of sound class models.
+
 The module provides functions for the customized compilation of sound-class models.
 All models are defined in simple text files. In order to guarantee their quick
 access when loading the library, the models are compiled and stored in binary
 files.
 """
-from __future__ import division,print_function
+__author__="Johann-Mattis List"
+__date__="2013-03-08"
+
 import networkx as nx
 from pickle import dump
 import os
+import codecs
 
+# lingpy imports
+from ..settings import rcParams
+try:
+    from ..algorithm.cython import misc
+except:
+    from ..algorithm.cython import _misc as misc
+from ..convert import *
+from ..read import *
 
 def _import_sound_classes(filename):
     """
@@ -17,7 +33,7 @@ def _import_sound_classes(filename):
     creates a replacement dictionary from these sound classes.
     """
 
-    infile = open(filename,'r')
+    infile = codecs.open(filename,'r','utf-8')
     data = []
     for line in infile:
         data.append(line.strip().split(' : '))
@@ -30,7 +46,7 @@ def _import_sound_classes(filename):
     
     for key in sc_dict.keys():
         for value in sc_dict[key]:
-            sc_repl_dict[unicode(value,'utf-8')] = key
+            sc_repl_dict[value] = key
 
     return sc_repl_dict
 
@@ -39,7 +55,7 @@ def _import_score_tree(filename):
     Function imports score trees for a given range of sound classes and
     converts them into a graph.
     """
-    infile = open(filename,'r')
+    infile = codecs.open(filename,'r','utf-8')
     graph = nx.DiGraph()
     data = []
     for line in infile:
@@ -318,8 +334,15 @@ def _make_scoring_dictionary(
     # the alignments getting worse, probably because most tests have been based
     # on profiles. we probably need a very good gap score. 
     for node in graph.nodes():
+        # missing data
+        score_dict[(node,'0')] = 0
+        score_dict[('0',node)] = 0
+
+        # swaps
         score_dict[(node,'+')] = -100
         score_dict[('+',node)] = -100
+
+        # specific values
         if graph.node[node]['val'] == 'v':
             score_dict[(node,'X')] = 0
             score_dict[('X',node)] = 0
@@ -329,9 +352,13 @@ def _make_scoring_dictionary(
         else:
             score_dict[(node,'X')] = 0
             score_dict[('X',node)] = 0
+    
     score_dict[('X','+')] = -5
     score_dict[('+','X')] = -5
     score_dict[('+','+')] = 0
+    score_dict[('0','0')] = 0
+    score_dict[('0','X')] = 0
+    score_dict[('X','0')] = 0
 
     # define the gaps
     score_dict[('X','X')] = 0
@@ -346,7 +373,7 @@ def _export_score_dict(score_dict):
     """
     
     letters = list(set([key[0] for key in score_dict.keys()]))
-    outfile = open('score_dict.csv','w')
+    outfile = codes.open('score_dict.csv','w','utf-8')
     outfile.write('\t'+'\t'.join(letters)+'\n')
     for letter1 in letters:
         outfile.write(letter1)
@@ -405,6 +432,18 @@ def compile_model(
             D : θ, ð, ŧ, þ, đ
             G : x, ɣ, χ
             ...    
+    
+    :file:`matrix`
+        A scoring matrix indicating the alignment scores of all sound-class
+        characters defined by the model. The scoring is structured as a simple
+        tab-delimited text file. The first cell contains the character names,
+        the following cells contain the scores in redundant form (with both
+        triangles being filled):: 
+
+            B  10.0 -10.0   5.0 ...
+            E -10.0   5.0 -10.0 ...
+            F   5.0 -10.0  10.0 ...
+            ...
 
     :file:`scorer` 
         The ``scorer`` file (which is optional) contains the graph of
@@ -442,42 +481,75 @@ def compile_model(
     print("[i] Compiling model <"+model+">...")
     # get the path to the models
     if not path:
-        path = os.path.split(os.path.abspath(__file__))[0]+'/models/'+model+'/'
+        new_path = os.path.join(
+                rcParams['_path'],
+                'data',
+                'models',
+                model
+                )
     else:
-        if path.endswith('/'):
-            path = path+model+'/'
-        else:
-            path = path + '/' + model + '/'
+        new_path = os.path.join(path,model)
     
+    if rcParams['debug']: print("Model-Path:",new_path)
+
     # load the sound classes
-    sound_classes = _import_sound_classes(path+'converter')
+    sound_classes = _import_sound_classes(os.path.join(new_path,'converter'))
     
     # dump the data
-    outfile = open(path+'converter.bin','wb')
+    outfile = open(os.path.join(new_path,'converter.bin'),'wb')
     dump(sound_classes,outfile)
     outfile.close()
     print("... successfully created the converter.")
 
-    # try to load the score tree
-    try:
-        score_tree = _import_score_tree(path+'scorer')
+    # try to load the scoring function or the score tree
+    scorer = False
+    
+    if os.path.isfile(os.path.join(new_path,'matrix')):
+        scorer = read_scorer(os.path.join(new_path,'matrix'))
+    elif os.path.isfile(os.path.join(new_path,'scorer')):
+        score_tree = _import_score_tree(os.path.join(new_path,'scorer'))
     
         # calculate the scoring dictionary
         score_dict = _make_scoring_dictionary(score_tree)
 
+        # make score_dict a ScoreDict instance
+
+        chars = sorted(set([s[0] for s in score_dict.keys()]))
+        matrix = [[0 for i in range(len(chars))] for j in
+                range(len(chars))]
+        for i,charA in enumerate(chars):
+            for j,charB in enumerate(chars):
+                if i < j:
+                    try:
+                        matrix[i][j] = score_dict[charA,charB]
+                        matrix[j][i] = score_dict[charB,charA]
+                    except KeyError:
+                        matrix[i][j] = -100
+                        matrix[j][i] = -100
+                elif i == j:
+                    matrix[i][j] = score_dict[charA,charB]        
+
+        scorer = misc.ScoreDict(chars,matrix)
+        
+        # create the matrix file
+        f = codecs.open(os.path.join(new_path,'matrix'),'w','utf-8')
+        f.write(scorer2str(scorer))
+        f.close()
+    
+    if scorer:
         # dump the data
-        outfile = open(path+'scorer.bin','wb')
-        dump(score_dict,outfile)
+        outfile = open(os.path.join(new_path,'scorer.bin'),'wb')
+        dump(scorer,outfile)
         outfile.close()
         print("... successfully created the scorer.")
-    except:
+    else:
         print("... no scoring dictionary defined.")
 
     print("[i] Model <"+model+"> was compiled successfully.")
 
-def compile_diacritics_and_vowels():
+def compile_dvt(path=''):
     """
-    Function compiles diacritics and vowels.
+    Function compiles diacritics, vowels, and tones.
 
     Notes
     -----
@@ -502,22 +574,45 @@ def compile_diacritics_and_vowels():
     print("[i] Compiling diacritics and vowels...")
 
     # get the path to the models
-    path = os.path.split(os.path.abspath(__file__))[0]+'/models/dv/'
+    if not path:
+        path = os.path.join(
+                rcParams['_path'],
+                'data',
+                'models',
+                'dvt'
+                )
+    elif path in ['evolaemp','el']:
+        path = os.path.join(
+                rcParams['_path'],
+                'data',
+                'models',
+                'dvt_el'
+                )
+    else:
+        pass
 
-    diacritics = open(
-            path+'diacritics',
-            'r'
+    diacritics = codecs.open(
+            os.path.join(path,'diacritics'),
+            'r',
+            'utf-8'
             ).read().replace('\n','').replace('-','')
-    vowels = open(
-            path+'vowels',
-            'r'
+    vowels = codecs.open(
+            os.path.join(path,'vowels'),
+            'r',
+            'utf-8'
             ).read().replace('\n','')
 
-    dv = (unicode(diacritics,'utf-8'),unicode(vowels,'utf-8'))
+    tones = codecs.open(
+            os.path.join(path,'tones'),
+            'r',
+            'utf-8'
+            ).read().replace('\n','')
 
-    outfile = open(path+'dv.bin','wb')
-    dump(dv,outfile)
+    dvt = (diacritics,vowels,tones)
+
+    outfile = open(os.path.join(path,'dvt.bin'),'wb')
+    dump(dvt,outfile)
     outfile.close()
 
-    print("[i] Diacritics and sound classes were successfully compiled.")
+    if rcParams['verbose']: print("[i] Diacritics and sound classes were successfully compiled.")
 
